#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

#from __future__ import print_function
from __future__ import division
from gengeo     import Sphere, Vector3, Line2D, BoxWithLines2D, MNTable2D, InsertGenerator2D
from random     import random

# An example python script to generate a bonded rectangle of particles
# and cluster the particles inside the rectangle 

# Define region extremities:
xsize=20.0
ysize=20.0
minPoint = Vector3(0.0,0.0,0.0)
maxPoint = Vector3(xsize,ysize,0.0)

# Define the geometrical constraints for packing
# 	(e.g. lines bordering a rectangular region in 2D)
top_line = Line2D (
   startPoint = Vector3(xsize,0.0,0.0),
   endPoint = minPoint
)

bottom_line = Line2D (
   startPoint = maxPoint,
   endPoint = Vector3(0.0,ysize,0.0)
)

left_line = Line2D (
   startPoint = Vector3(xsize,0.0,0.0),
   endPoint = maxPoint
)

right_line = Line2D (
   startPoint = minPoint,
   endPoint = Vector3(0.0,ysize,0.0)
)

# Define the Volume to be filled with spheres:
box = BoxWithLines2D (
   minPoint = minPoint,
   maxPoint = maxPoint
)

box.addLine(top_line)
box.addLine(bottom_line)
box.addLine(left_line)
box.addLine(right_line)

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable2D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5,
   numGroups = 2
)

# Fill the volume with particles:
packer = InsertGenerator2D (
   minRadius = 0.1,
   maxRadius = 1.0,
   insertFails = 1000,
   maxIterations = 1000,
   tolerance = 1.0e-6,
   seed=0
)

packer.generatePacking( volume = box, ntable = mntable, groupID = 0, tag = 0)

# insert  zero-radius particles in group 2 as a cluster seeds
# use randomly disturbed square grid
nx=5
ny=5
for i in range(nx):
    for j in range(ny):
        xpos=(float(i)+random())*xsize/float(nx)
        ypos=(float(j)+random())*ysize/float(ny)
        cseed=Sphere(Vector3(xpos,ypos,0.0),0.0)
#        cseed.setTag(i*ny+j)
        cseed.setTag(int(random()*nx*ny))
        mntable.insert(
           sphere=cseed,
           groupID=1
        )
        #print("xpos: ",xpos)

mntable.tagParticlesToClosest(
   groupID1=0,
   groupID2=1
)

# create cluster bonds between neighbouring particles
# bonds between particles with the same tag will have bondTag1
# bonds between particles with different tags will have bondTag2
mntable.generateClusterBonds(
   groupID = 0,
   tolerance = 1.0e-5,
   bondTag1 = 0,
   bondTag2 = 1,
)


# write a geometry file
mntable.write(
   fileName = "temp/cluster.geo",
   outputStyle = 1
)

# write a vtk file
mntable.write(
   fileName = "temp/cluster.vtu",
   outputStyle = 2
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from __future__ import division
from gengeo     import *
#An example python script to generate a triangular prism of bonded particles

size=5.0
# Define region extremities:
minPoint = Vector3(-1.0*size,-1.0*size,-1.0*size)
maxPoint = Vector3(size,size,size)

# Define the volume to be filled with spheres:
tribox = TriBox (
   minPoint = minPoint,
   maxPoint = maxPoint,
   inverted = False
)
tribox.addPlane(Plane(minPoint,Vector3(0.0,1.0,0.0)))
tribox.addPlane(Plane(minPoint,Vector3(0.0,0.0,1.0)))
tribox.addPlane(Plane(maxPoint,Vector3(0.0,0.0,-1.0)))

delta = maxPoint-minPoint
dx = delta.X()
dy = delta.Y()
tribox.addPlane(Plane(minPoint, Vector3(dy,-0.5*dx,0.0).unit()))
tribox.addPlane(Plane(maxPoint-Vector3(dx/2.0,0.0,0.0), Vector3(-1.0*dy,-0.5*dx,0.0).unit()))

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable3D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5
)

# Fill the volume with particles:
packer = InsertGenerator3D (
   minRadius = 0.2,
   maxRadius = 1.0,
   insertFails = 1000,
   maxIterations = 1000,
   tolerance = 1.0e-6
)

# Generate the packing
packer.generatePacking(
   volume = tribox, 
   ntable = mntable
)

# generate bonds between neighbouring particles
mntable.generateBonds(
   tolerance = 1.0e-5,
   bondID = 0
)

# write a geometry file in VTK format
mntable.write(
   fileName = "temp/geo_example6.vtu",
   outputStyle = 2
)

# write a geometry file in ESyS-Particle geo format
mntable.write(
   fileName = "temp/geo_example6.geo",
   outputStyle = 1
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

# Init file for gengeo package.

from gengeo import *



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import *

xsize=50.0
ysize=50.0
ysize_cl=10.0
ntry=1000
rmin_nb=0.2
rmin_b=0.2
pbond=0.5

y_bdry_1=0.5*(ysize-ysize_cl)
y_bdry_2=ysize-y_bdry_1

T = MNTable2D(Vector3(0.0,0.0,0.0),Vector3(xsize,ysize,0.0),2.5,1)
GB = InsertGenerator2D(rmin_b,1.0,ntry,1000,1e-6)
GNB = InsertGenerator2D(rmin_nb,1.0,ntry,1000,1e-6)

CenterBox = BoxWithLines2D (Vector3(0.0,y_bdry_1,0.0),Vector3(xsize,y_bdry_2,0.0))

CenterBox.addLine(Line2D(Vector3(0.0,y_bdry_1,0.0),Vector3(0.0,y_bdry_2,0.0)))
CenterBox.addLine(Line2D(Vector3(xsize,y_bdry_1,0.0),Vector3(xsize,y_bdry_2,0.0)))
CenterBox.addLine(Line2D(Vector3(0.0,y_bdry_1,0.0),Vector3(xsize,y_bdry_1,0.0)))
CenterBox.addLine(Line2D(Vector3(0.0,y_bdry_2,0.0),Vector3(xsize,y_bdry_2,0.0)))

GB.generatePacking(CenterBox,T,0,1)
T.generateRandomBonds(0,1e-5,pbond,0,1,0)

BottomBox = BoxWithLines2D (Vector3(0.0,0.0,0.0),Vector3(xsize,y_bdry_1+0.5,0.0))

BottomBox.addLine(Line2D(Vector3(0.0,0.0,0.0),Vector3(0.0,y_bdry_1,0.0)))
BottomBox.addLine(Line2D(Vector3(xsize,0.0,0.0),Vector3(xsize,y_bdry_1,0.0)))
BottomBox.addLine(Line2D(Vector3(0.0,0.0,0.0),Vector3(xsize,0.0,0.0)))

GNB.generatePacking(BottomBox,T,0,2)

TopBox = BoxWithLines2D (Vector3(0.0,y_bdry_2-0.5,0.0),Vector3(xsize,ysize,0.0))

TopBox.addLine(Line2D(Vector3(0.0,y_bdry_2,0.0),Vector3(0.0,ysize,0.0)))
TopBox.addLine(Line2D(Vector3(xsize,y_bdry_2,0.0),Vector3(xsize,ysize,0.0)))
TopBox.addLine(Line2D(Vector3(0.0,ysize,0.0),Vector3(xsize,ysize,0.0)))

GNB.generatePacking(TopBox,T,0,3)

T.write("temp/geo_hlayer.vtu",2)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import Vector3, Line2D, BoxWithLines2D, MNTable2D, InsertGenerator2D, CircleVol

# An example python script to generate a 2D rectangle of particles
# containing a circular cluster of bonded particles

# Define region extremities:
minPoint = Vector3(0.0,0.0,0.0)
maxPoint = Vector3(10.0,10.0,0.0)

# Define the volume to be filled with spheres:
box = BoxWithLines2D (
   minPoint = minPoint,
   maxPoint = maxPoint
)

# Define the geometrical constraints for packing
top_line = Line2D (
   startPoint = Vector3(10.0,0.0,0.0),
   endPoint = minPoint
)

bottom_line = Line2D (
   startPoint = maxPoint,
   endPoint = Vector3(0.0,10.0,0.0)
)

left_line = Line2D (
   startPoint = Vector3(10.0,0.0,0.0),
   endPoint = maxPoint
)

right_line = Line2D (
   startPoint = minPoint,
   endPoint = Vector3(0.0,10.0,0.0)
)


box.addLine(top_line)
box.addLine(bottom_line)
box.addLine(left_line)
box.addLine(right_line)

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable2D (
   minPoint = minPoint,		# specific regional extremities
   maxPoint = maxPoint,
   gridSize = 2.5		# neighbour search algorithm grid size
)

# Fill the volume with particles:
packer = InsertGenerator2D (
   minRadius = 0.1,		# minimum radius of spheres to insert
   maxRadius = 0.4,		# maximum radius of spheres to insert
   insertFails = 5000,		# maximum number of insertion failures
   maxIterations = 1000,	# max. iterations for Newton-Raphson solver
   tolerance = 1.0e-6		# max. allowable overlap of spheres
)

packer.generatePacking( volume = box, ntable = mntable, tag = 0)

# define circular area
center = Vector3(5.0,5.0,0.0)
radius = 2.5

circ_area=CircleVol(center,radius)

# tag particles in circular area
mntable.tagParticlesInVolume(circ_area,2,0)


# create bonds between neighbouring particles with tag 2:
mntable.generateBondsWithMask(
   tolerance = 1.0e-5,	# max. gap between "touching" particles
   bondID = 0,		# bond ID of created bonds
   tag = 2,
   mask = 2
)

# write the geometry to a file in raw format
mntable.write(
   fileName = "temp/geo_example11.txt",
   outputStyle = 0
)

# write the geometry to an ESyS-Particle geometry file
mntable.write(
   fileName = "temp/geo_example11.geo",
   outputStyle = 1
)

# write the geometry as a VTK file
mntable.write(
   fileName = "temp/geo_example11.vtu",
   outputStyle = 2
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import *
#An example python script to generate a union/intersection of two volumes 

# Define region extremities:
maxRadius = 1.0
size = 20.0
minPoint = Vector3(0.0,0.0,0.0)
maxPoint = Vector3(size,size,size)

minPt = Vector3(0,0,0)
maxPt = Vector3(size,0.5*size,size)
# Define the first volume to be filled with spheres:
box = BoxWithPlanes3D (
   minPoint = minPt,
   maxPoint = maxPt
)

box.addPlane(
   Plane(
      origin = minPt, 
      normal = Vector3(1.0,0.0,0.0)
   )
)
#or the compact form:
box.addPlane(Plane(minPt, Vector3(0.0,1.0,0.0)))
box.addPlane(Plane(minPt, Vector3(0.0,0.0,1.0)))
box.addPlane(Plane(maxPt, Vector3(-1.0,0.0,0.0)))
box.addPlane(Plane(maxPt, Vector3(0.0,-1.0,0.0)))
box.addPlane(Plane(maxPt, Vector3(0.0,0.0,-1.0)))

sphere = SphereVol (
   centre = Vector3(0.5*size,0.5*size,0.5*size),
   radius = 0.5*size
)

sphere1 = SphereVol (
   centre = Vector3(0.5*size,0.75*size,0.5*size),
   radius = 0.5*size
)

sphere2 = SphereVol (
   centre = Vector3(0.5*size,0.25*size,0.5*size),
   radius = 0.5*size
)

cylinder = CylinderVol (
   origin = Vector3 (0.5*size,0,0.5*size),
   axis = Vector3 (0,1,0),
   length = 0.5*size,
   radius = 0.5*size
)

cylinder2 = CylinderVol (
   origin = Vector3 (0.5*size,0,0.5*size),
   axis = Vector3 (0,1,0),
   length = 0.5*size,
   radius = 0.4*size
)

cylinder3 = CylinderVol (
   origin = Vector3 (0.5*size,0.25*size,0.5*size),
   axis = Vector3 (0,1,0),
   length = 0.5*size,
   radius = 0.3*size
)

unionVol = UnionVol (
   volume1 = sphere,
   volume2 = cylinder
)

intersectionVol = IntersectionVol (
   volume1 = sphere1,
   volume2 = sphere2
)

differenceVol = DifferenceVol (
   volume1 = cylinder,
   volume2 = cylinder2
)

differenceVol2 = DifferenceVol (
   volume1 = cylinder,
   volume2 = sphere1
)

differenceVol3 = DifferenceVol (
   volume1 = cylinder,
   volume2 = cylinder3
)

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable3D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5*maxRadius
)

# Fill the volume with particles:
packer = InsertGenerator3D (
   minRadius = 0.2,
   maxRadius = maxRadius,
   insertFails = 1000,
   maxIterations = 1000,
   tolerance = 1.0e-6
)

packer.generatePacking(
#   volume = unionVol, 
#   volume = intersectionVol, 
#   volume = differenceVol, 
   volume = differenceVol2, 
#   volume = differenceVol3, 
#   volume = cylinder, 
   ntable = mntable
)

# create bonds between neighbouring particles:
mntable.generateBonds(
   tolerance = 1.0e-5,
   bondID = 0
)

mntable.tagParticlesInVolume (
   volume = differenceVol,
   tag = 12345
)

# write a geometry file
mntable.write(
   fileName = "temp/geo_example10.vtu",
   outputStyle = 2	
)

mntable.write(
   fileName = "temp/geo_example10.raw",
   outputStyle = 0
)

mntable.write(
   fileName = "temp/geo_example10.geo",
   outputStyle = 1	
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from __future__ import division
from gengeo     import *

def generate_upper_tri_rough_block(NT, G, xt, yt, zt, y0, nr, yr):
   min = Vector3(0.0,y0,0.0)
   min_2 = Vector3(0.0,y0+yr,0.0)
   max = Vector3(xt,y0+yt,zt)
   xr = xt/(1.0*nr)

   top_plane = Plane (max, Vector3(0,-1,0))
   bottom_plane_1 = Plane(min, Vector3(0,1,0))
   bottom_plane_2 = Plane(min_2, Vector3(0,1,0))
   front_plane = Plane(min, Vector3(0,0,1))
   back_plane = Plane(max, Vector3(0,0,-1))

   diff = Vector3(1,0,0)

   Box = BoxWithPlanes3D (min_2-diff,max+diff)

   Box.addPlane(top_plane)
   Box.addPlane(bottom_plane_2)
   Box.addPlane(front_plane)
   Box.addPlane(back_plane)

   G.generatePacking(Box,NT,0)

   for i in range(0,nr):
      x1 = (1.0*i)*xr
      x2 = (1.0*i+1.0)*xr

      lmin = Vector3(x1,y0,0.0)
      lmax = Vector3(x2,y0+yr,zt)

      left_plane = Plane (lmin+Vector3(xr/2.0,0.0,0.0), (Vector3(yr,-0.5*xr,0.0)).unit())
      right_plane = Plane (lmax, Vector3(-1.0*yr,-0.5*xr,0.0).unit())

      RBox = TriBox (lmin,lmax,True)
      RBox.addPlane(front_plane)
      RBox.addPlane(back_plane)
      RBox.addPlane(left_plane)
      RBox.addPlane(right_plane)

      G.generatePacking(RBox, NT, 0)

#   NT.tagParticlesAlongPlane(top_plane,0.5,3,0)
   NT.tagParticlesAlongPlane(top_plane,yt,3,0)

def generate_lower_tri_rough_block(NT, G, xt, yt, zt, y0, nr, yr):
   min = Vector3(0.0,y0,0.0)
   max_2 = Vector3(xt,y0+(yt-yr),zt)
   max = Vector3(xt,y0+yt,zt)
   xr = xt/(nr)

   top_plane = Plane (max, Vector3(0,-1,0))
   top_plane_2 = Plane(max_2, Vector3(0,-1,0))
   bottom_plane = Plane(min, Vector3(0,1,0))
   front_plane = Plane(min, Vector3(0,0,1))
   back_plane = Plane(max, Vector3(0,0,-1))

   diff = Vector3(1,0,0)

   Box = BoxWithPlanes3D (min-diff,max_2+diff)

   Box.addPlane(top_plane_2)
   Box.addPlane(bottom_plane)
   Box.addPlane(front_plane)
   Box.addPlane(back_plane)

   G.generatePacking(Box,NT,0)

   for i in range(0,nr):
      x1 = (1.0*i)*xr
      x2 = (1.0*i+1.0)*xr

      lmin = Vector3(x1,y0+(yt-yr),0.0)
      lmax = Vector3(x2,y0+yt,zt)

      left_plane = Plane (lmin, Vector3(yr,-0.5*xr,0).unit())
      right_plane = Plane (lmax-Vector3(xr/2.0,0.0,0.0), Vector3(-1.0*yr,-0.5*xr,0.0).unit())

      RBox = TriBox (lmin,lmax,False)
      RBox.addPlane(front_plane)
      RBox.addPlane(back_plane)
      RBox.addPlane(left_plane)
      RBox.addPlane(right_plane)

      G.generatePacking(RBox, NT, 0)

#   NT.tagParticlesAlongPlane(bottom_plane,0.5,4,0)
   NT.tagParticlesAlongPlane(bottom_plane,yt,4,0)

def generate_granular_gouge(NT, G, Pmin, Pmax, grmin, grmax, gntry):
   T_aux = MNTable3D (Pmin, Pmax, 2.1*grmax, 1)
   G_aux = InsertGenerator3D (grmin, grmax, gntry, 1000, 1.0e-5)
   V_aux = BoxWithPlanes3D (Pmin, Pmax)

   G_aux.generatePacking(V_aux, T_aux, 0)

   sphere_list = T_aux.getSphereListFromGroup(0)

   ngr = len(sphere_list)
   NT.GrowNGroups(1+ngr)
   ggid = 1
   for ss in sphere_list:
      S = SphereVol(ss.Centre(), ss.Radius())
      G.generatePacking(S,NT,ggid)
      NT.generateBonds(ggid,1.0e-5,1)
      ggid += 1



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import *
#An example python script to generate a dogbone of unbonded particles

# Define region extremities:
l2 = 10.0
r2 = 2.5
origin = Vector3(0.0,0.0,0.0)
axis = Vector3(0.0,1.0,0.0)
size = 5.0
minPoint = Vector3(-1.0*size,0.0,-1.0*size)
maxPoint = Vector3(size,3.0*size,size)

# Define the volume to be filled with spheres:
dogbone = DogBone (
   origin = origin,
   axis = axis,
   length = 3.0*size,
   radius = size,
   l2 = l2,		
   r2 = r2
)

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable3D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.2
)

# Fill the volume with particles:
packer = InsertGenerator3D (
   minRadius = 0.2,
   maxRadius = 1.0,
   insertFails = 1000,
   maxIterations = 1000,
   tolerance = 1.0e-6
)

# Generate the packing
packer.generatePacking(
   volume = dogbone, 
   ntable = mntable
)

# write a geometry file in VTK format
mntable.write(
   fileName = "temp/geo_example4.vtu",
   outputStyle = 2		
)

# write a geometry file in ESyS-Particle geo format
mntable.write(
   fileName = "temp/geo_example4.geo",
   outputStyle = 1		
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import *
from granular_gouge import *

xsize = 20.0
ysize = 20.0
ysize_bdry = 4.0
zsize = 20.0
numRidges = 5
heightRidges = 2.5
numInsertFails = 1000
minRadius = 0.2
minGrainRadius = 2.0
maxGrainRadius = 4.0

minPoint = Vector3(0,0,0)
maxPoint = Vector3(xsize,ysize,zsize)
minGPoint = Vector3(0,ysize_bdry,0)
maxGPoint = Vector3(xsize,ysize-ysize_bdry,zsize)

mntable = CircMNTable3D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5,
   numGroups = 3
)

packer = InsertGenerator3D (
   minRadius = minRadius,
   maxRadius = 1.0,
   insertFails = numInsertFails,
   maxIterations = 1000,
   tolerance = 1.0e-6
)

generate_upper_tri_rough_block(mntable, packer, xsize, ysize_bdry, zsize, ysize-ysize_bdry, numRidges, heightRidges)

generate_lower_tri_rough_block(mntable, packer, xsize, ysize_bdry, zsize, 0.0, numRidges, heightRidges)

generate_granular_gouge(mntable, packer, minGPoint, maxGPoint, minGrainRadius, maxGrainRadius, numInsertFails)

mntable.generateBonds(
   tolerance = 1.0e-5,
   bondID = 0
)

mntable.write(
   fileName = "temp/geo_gouge.vtu",
   outputStyle = 2   
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from __future__ import division
from gengeo     import *

size = 50.0
ntry = 1000
rmin_sand = 0.5
rmin_clay = 0.2
rmax_clay = 0.5

sand_layer_thickness = 0.425*size
clay_layer_thickness = 0.15*size
center_box_dim = 0.05*size

mntable = CircMNTable2D (Vector3(0,0,0), Vector3(size,size,0),2.5,1)

sand_packer = HexAggregateInsertGenerator2D (rmin_sand, 1.0, ntry, 1000, 1.0e-6)

clay_packer = InsertGenerator2D(rmin_clay, rmax_clay, ntry, 1000, 1.0e-6)

clay_layer_min=sand_layer_thickness
clay_layer_max=clay_layer_min+clay_layer_thickness
clay_box_min = clay_layer_min - rmax_clay
clay_box_max = clay_layer_max + rmax_clay

ClayBox = BoxWithLines2D(Vector3(clay_layer_min,0.0,0.0),Vector3(clay_layer_max,size,0.0))

ClayBox.addLine(Line2D(Vector3(clay_layer_min,0.0,0.0),Vector3(clay_layer_max,0.0,0.0)))
ClayBox.addLine(Line2D(Vector3(clay_layer_max,size,0.0),Vector3(clay_layer_min,size,0.0)))
ClayBox.addLine(Line2D(Vector3(clay_layer_max,size,0.0),Vector3(clay_layer_max,0.0,0.0)))
ClayBox.addLine(Line2D(Vector3(clay_layer_min,size,0.0),Vector3(clay_layer_max,size,0.0)))

clay_packer.generatePacking(ClayBox,mntable,0,2)

cbox_layer_minx=sand_layer_thickness-center_box_dim
cbox_layer_maxx=sand_layer_thickness+clay_layer_thickness+center_box_dim
cbox_layer_miny=0.5*size-center_box_dim
cbox_layer_maxy=0.5*size+center_box_dim

CenterBox = BoxWithLines2D(Vector3(cbox_layer_minx,cbox_layer_miny,0.0), Vector3(cbox_layer_maxx,cbox_layer_maxy,0.0))

CenterBox.addLine(Line2D(Vector3(cbox_layer_minx,cbox_layer_miny,0.0), Vector3(cbox_layer_maxx,cbox_layer_miny,0.0)))

CenterBox.addLine(Line2D(Vector3(cbox_layer_maxx,cbox_layer_maxy,0.0), Vector3(cbox_layer_minx,cbox_layer_maxy,0.0)))

CenterBox.addLine(Line2D(Vector3(cbox_layer_maxx,cbox_layer_maxy,0.0), Vector3(cbox_layer_maxx,cbox_layer_miny,0.0)))

CenterBox.addLine(Line2D(Vector3(cbox_layer_minx,cbox_layer_maxy,0.0), Vector3(cbox_layer_maxx,cbox_layer_maxy,0.0)))

clay_packer.generatePacking(CenterBox,mntable,0,2)

sand_layer_min1=0
sand_layer_max1=sand_layer_thickness
sand_box_min1=sand_layer_min1-rmin_sand
sand_box_max1=sand_layer_max1+rmin_sand

SandBox1 = BoxWithLines2D(Vector3(sand_box_min1,0.0,0.0),Vector3(sand_box_max1,size,0.0))

SandBox1.addLine(Line2D(Vector3(sand_layer_min1,size,0.0),Vector3(sand_layer_min1,0.0,0.0)))

SandBox1.addLine(Line2D(Vector3(sand_layer_min1,0.0,0.0),Vector3(sand_layer_max1,0.0,0.0)))

SandBox1.addLine(Line2D(Vector3(sand_layer_max1,size,0.0),Vector3(sand_layer_max1,0.0,0.0)))

SandBox1.addLine(Line2D(Vector3(sand_layer_min1,size,0.0),Vector3(sand_layer_max1,size,0.0)))

SandBox1.addLine(Line2D(Vector3(0.0,size/2.0,0.0),Vector3(size,size/2.0,0.0)))

sand_packer.generatePacking(SandBox1,mntable,0,1)

sand_layer_min2=sand_layer_thickness+clay_layer_thickness
sand_layer_max2=size
sand_box_min2=sand_layer_min2-rmin_sand
sand_box_max2=sand_layer_max2+rmin_sand

SandBox2 = BoxWithLines2D(Vector3(sand_box_min2,0.0,0.0),Vector3(sand_box_max2,size,0.0))

SandBox2.addLine(Line2D(Vector3(sand_layer_min2,size,0.0),Vector3(sand_layer_min2,0.0,0.0)))
SandBox2.addLine(Line2D(Vector3(sand_layer_min2,0.0,0.0),Vector3(sand_layer_max2,0.0,0.0)))
SandBox2.addLine(Line2D(Vector3(sand_layer_max2,size,0.0),Vector3(sand_layer_max2,0.0,0.0)))
SandBox2.addLine(Line2D(Vector3(sand_layer_min2,size,0.0),Vector3(sand_layer_max2,size,0.0)))

SandBox2.addLine(Line2D(Vector3(0.0,size/2.0,0.0),Vector3(size,size/2.0,0.0)))

sand_packer.generatePacking(SandBox2,mntable,0,1)

mntable.generateBondsWithMask(0,1.0e-6,0,1,1)

top_line = Line2D(Vector3(0.0,size,0.0),Vector3(size,size,0.0))
bottom_line = Line2D(Vector3(0.0,0.0,0.0),Vector3(size,0.0,0.0))

mntable.tagParticlesAlongLineWithMask(top_line,0.5,4,4,0)
mntable.tagParticlesAlongLineWithMask(bottom_line,0.5,8,8,0)

mntable.write("temp/geo_layered.vtu",2)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import *
from math import sqrt
#An example python script to generate a regular aggregate of hexagonal grains
# with bonds broken along lines 

# Define region extremities:
size = 150.0
xsize = size
ysize = size
minPoint = Vector3(0.0,0.0,0.0)
maxPoint = Vector3(xsize,ysize,0.0)

# Define the geometrical constraints for packing
top_line = Line2D (
   startPoint = minPoint,
   endPoint = Vector3(size,0.0,0.0)
)

bottom_line = Line2D (
   startPoint = Vector3(0.0,40.,0.0),
   endPoint = Vector3(40.,40.,0.0)
)

#or compact form:
left_line = Line2D(Vector3(40.,40.,0.0),Vector3(40.,0.0,0.0))
right_line = Line2D(minPoint, Vector3(0.0,ysize,0.0))

box = BoxWithLines2D (
   minPoint = minPoint,
   maxPoint = maxPoint
)
box.addLine(top_line)
box.addLine(bottom_line)
box.addLine(left_line)
box.addLine(right_line)

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable2D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 1.25
)

# Fill the volume with particles:
packer = HGrainGenerator2D (
   radius = 0.5
)

# Generate the packing
packer.generatePacking(
   volume = box, 
   ntable = mntable,
   tag = 0
)

# bond the particles
#mntable.generateBonds(0,1.0e-5,3)

# tag particles along the boundary lines (e.g. top_line, left_line etc.)
#	using the compact form:
mntable.tagParticlesAlongLineWithMask(bottom_line, 5.0, 4, -1, 0)
mntable.tagParticlesAlongLineWithMask(top_line, 5.0, 8, -1, 0)
mntable.tagParticlesAlongLineWithMask(left_line, 8.0, 16, -1, 0)
mntable.tagParticlesAlongLineWithMask(right_line, 8.0, 32,-1, 0)

# write a geometry file in VTK format
mntable.write(
   fileName = "temp/geo_example8.vtu",
   outputStyle = 2
)

# write a geometry file in ESyS-Particle geo format
mntable.write(
   fileName = "temp/geo_example8.geo",
   outputStyle = 1
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from __future__ import division, print_function
from gengeo     import *
#An example python script to generate an aggregate of hexagonal grains
# with circular boundary conditions and tagging of particles along lines

# Define region extremities:
size = 5.0
minPoint = Vector3(0.0,0.0,0.0)
maxPoint = Vector3(size,size,0.0)

# Define the geometrical constraints for packing
top_line = Line2D (
   startPoint = Vector3(size,0.0,0.0),
   endPoint = minPoint
)

bottom_line = Line2D (
   startPoint = maxPoint,
   endPoint = Vector3(0.0,size,0.0)
)

box = BoxWithLines2DSubVol (
   minPoint = minPoint-Vector3(-0.5,0.0,0.0),
   maxPoint = maxPoint+Vector3(0.5,0.0,0.0),
   svdim_x = 2.5,
   svdim_y = 2.5
)
box.addLine(top_line)
box.addLine(bottom_line)

# Create a multi-group neighbour table to contain the particles:
mntable = CircMNTable2D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5
)

# Fill the volume with particles:
packer = HexAggregateInsertGenerator2D (
   minRadius = 0.1,
   maxRadius = 1.0,
   insertFails = 1000,
   maxIterations = 1000,
   tolerance = 1.0e-6
)

# Generate the packing
packer.generatePacking(
   volume = box, 
   ntable = mntable,
   tag = 0
)

# tag particles along the top_line and bottom_line
mntable.tagParticlesAlongLineWithMask(
   line = top_line,
   distance = 0.5,
   tag = 4,
   mask = 4
)

#and the compact form:
mntable.tagParticlesAlongLineWithMask(bottom_line, 0.5, 8, 8, 0)

#print the porosity of the particle packing:
volume = size*size
porosity = (volume - mntable.getSumVolume())/volume
print("Porosity:  ",porosity)

# write a geometry file in VTK format
mntable.write(
   fileName = "temp/geo_example7.vtu",
   outputStyle = 2
)

# write a geometry file in ESyS-Particle geo format
mntable.write(
   fileName = "temp/geo_example7.geo",
   outputStyle = 1
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import *
from random import random

#An example python script to generate a bonded rectangle of particles with a discrete fracture network included

width = 50.0
height = width
# Define region extremities:
minPoint = Vector3(0.0,0.0,0.0)
maxPoint = Vector3(width,height,0.0)

# Define the geometrical constraints for packing
# 	(e.g. lines bordering a rectangular region in 2D)
# QUESTION: Is there a particular order for defining endpoints of lines?
top_line = Line2D (
   startPoint = Vector3(width,0.0,0.0),
   endPoint = minPoint
)

bottom_line = Line2D (
   startPoint = maxPoint,
   endPoint = Vector3(0.0,height,0.0)
)

left_line = Line2D (
   startPoint = Vector3(width,0.0,0.0),
   endPoint = maxPoint
)

right_line = Line2D (
   startPoint = minPoint,
   endPoint = Vector3(0.0,height,0.0)
)

# Define the Volume to be filled with spheres:
#	(e.g. a BoxWithLines2D)
box = BoxWithLines2D (
   minPoint = minPoint,
   maxPoint = maxPoint
)

box.addLine(top_line)
box.addLine(bottom_line)
box.addLine(left_line)
box.addLine(right_line)


# Create a multi-group neighbour table to contain the particles:
mntable = MNTable2D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5
)

# Fill the volume with particles:
packer = InsertGenerator2D (
   minRadius = 0.1,
   maxRadius = 1.0,
   insertFails = 10000,
   maxIterations = 10000,
   tolerance = 1.0e-6
)

packer.generatePacking( volume = box, ntable = mntable, tag = 0)

# create bonds between neighbouring particles:
mntable.generateBonds(
   tolerance = 1.0e-5,
   bondID = 0
)

#Add a discrete fracture network
n_faults = 10
breakLines = []
for f in range(n_faults):

   x0 = random()*0.8*width + 0.1*width
   y0 = random()*0.8*height + 0.1*height
   x1 = random()*0.8*width + 0.1*width
   y1 = random()*0.8*height + 0.1*height

   brkLine = LineSegment2D (
      startPoint = Vector3(x0,y0,0.0),
      endPoint = Vector3(x1,y1,0.0)
   )
   breakLines.append(brkLine)

   mntable.breakBondsAlongLineSegment(
      lineSegment = brkLine,
      distance = 0.25,
      tag = 0
   )

# write a geometry file
mntable.write(
   fileName = "temp/geo_DFN_2.vtu",
   outputStyle = 2
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from gengeo import *

# Define region extremities:
minPoint = Vector3(0.0,25.0,0.0)
maxPoint = Vector3(30.0,40.0,0.0)

top_line = Line2D (
   startPoint = Vector3(30.0,25.0,0.0),
   endPoint = minPoint
)

bottom_line = Line2D (
   startPoint = maxPoint,
   endPoint = Vector3(0.0,40.0,0.0)
)

left_line = Line2D (
   startPoint = Vector3(30.0,25.0,0.0),
   endPoint = maxPoint
)

right_line = Line2D (
   startPoint = minPoint,
   endPoint = Vector3(0.0,40.0,0.0)
)

box = BoxWithLines2D (
   minPoint = minPoint,
   maxPoint = maxPoint
)

box.addLine(top_line)
box.addLine(bottom_line)
box.addLine(left_line)
box.addLine(right_line)

mntable = MNTable2D (
   minPoint = Vector3(0.0,0.0,0.0),
   maxPoint = Vector3(30.0,40.0,0.0),
   gridSize = 1.25
)

packer = InsertGenerator2D (
   minRadius = 0.4,
   maxRadius = 0.6,
   insertFails = 5000,
   maxIterations = 10000,
   tolerance = 1.0e-6
)

packer.generatePacking( volume = box, ntable = mntable, tag = 0)

#tri = PolygonWithLines2D (
#   centre = Vector3(15.0,15.0,0.0),
#   radius = 10.0,
#   nsides = 3,
#   smooth_edges = True
#)

left_tri = TriWithLines2D (
   vertex0 = Vector3(0.0,0.0,0.0),
   vertex1 = Vector3(10.0,0.0,0.0),
   vertex2 = Vector3(0.0,17.0,0.0)
)

right_tri = TriWithLines2D (
   vertex0 = Vector3(20.0,0.0,0.0),
   vertex1 = Vector3(30.0,0.0,0.0),
   vertex2 = Vector3(30.0,17.0,0.0)
)

obstacle = BoxWithLines2D (
   minPoint = Vector3(10.0,15.0,0.0),
   maxPoint = Vector3(20.0,23.0,0.0)
)

t_line = Line2D (
   startPoint = Vector3(10.0,23.0,0.0),
   endPoint = Vector3(20.0,23.0,0.0)
)
obstacle.addLine(t_line)

b_line = Line2D (
   startPoint = Vector3(10.0,15.0,0.0),
   endPoint = Vector3(20.0,15.0,0.0)
)
obstacle.addLine(b_line)

l_line = Line2D (
   startPoint = Vector3(10.0,15.0,0.0),
   endPoint = Vector3(10.0,23.0,0.0)
)
obstacle.addLine(l_line)

r_line = Line2D (
   startPoint = Vector3(20.0,15.0,0.0),
   endPoint = Vector3(20.0,23.0,0.0)
)
obstacle.addLine(r_line)

packer2 = InsertGenerator2D (
   minRadius = 0.1,
   maxRadius = 1.0,
   insertFails = 5000,
   maxIterations = 10000,
   tolerance = 1.0e-6
)

packer2.generatePacking( volume = left_tri, ntable = mntable, tag = 999)
packer2.generatePacking( volume = right_tri, ntable = mntable, tag = 999)
packer2.generatePacking( volume = obstacle, ntable = mntable, tag = 999)

# write a geometry file
mntable.write(
   fileName = "temp/obstructed_flow.vtu",
   outputStyle = 2
)

mntable.write(
   fileName = "temp/obstructed_flow.geo",
   outputStyle = 1
)

mntable.write(
   fileName = "temp/obstructed_flow.raw",
   outputStyle = 0
)



#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from __future__ import division, print_function
from gengeo     import *
#An example python script to generate a bonded rectangular prism

# Define region extremities:
maxRadius = 1.0
size = 4.0
minPoint = Vector3(0.0,0.0,0.0)
maxPoint = Vector3(size,2.0*size,size)

# Define the volume to be filled with spheres:
# 	(e.g. a box bounded by planes)
# QUESTION: Are there constraints on normals e.g. inward facing?
box = BoxWithPlanes3D (
   minPoint = minPoint,
   maxPoint = maxPoint
)

box.addPlane(
   Plane(
      origin = minPoint, 
      normal = Vector3(1.0,0.0,0.0)
   )
)
#or the compact form:
box.addPlane(Plane(minPoint, Vector3(0.0,1.0,0.0)))
box.addPlane(Plane(minPoint, Vector3(0.0,0.0,1.0)))
box.addPlane(Plane(maxPoint, Vector3(-1.0,0.0,0.0)))
box.addPlane(Plane(maxPoint, Vector3(0.0,-1.0,0.0)))
box.addPlane(Plane(maxPoint, Vector3(0.0,0.0,-1.0)))

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable3D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5*maxRadius
)


sList = ShapeList()

sList.addGenericShape(
   db = "shapeDatabase.db",
   name = "sphere",
   bias = 1,
   random = 1,
   particleTag = 1,
   bondTag = 1
)

sList.addGenericShape(
   db = "shapeDatabase.db",
   name = "test",
   bias = 5,
   random = 1,
   particleTag = 2,
   bondTag = 2 
)
   
# Fill the volume with particles:
packer = InsertGenerator3D (
   minRadius = 0.2,
   maxRadius = maxRadius,
   insertFails = 1000,
   maxIterations = 1000,
   tolerance = 1.0e-6
)

packer.generatePacking(
   volume = box, 
   ntable = mntable,
   tag = 0,
   shapeList = sList
)

# print the porosity:
volume = 2.0*size*size*size
porosity = (volume - mntable.getSumVolume())/volume
print("Porosity:  ", porosity)

# write a geometry file in VTK format
mntable.write(
   fileName = "temp/geo_exampleShapeList.vtu",
   outputStyle = 2	
)

# write a geometry file in raw (debug) format
mntable.write(
   fileName = "temp/geo_exampleShapeList.raw",
   outputStyle = 0		
)

# write a geometry file in gengeo file format
mntable.write(
   fileName = "temp/geo_exampleShapeList.geo",
   outputStyle = 1                      
)




#############################################################
##                                                         ##
## Copyright (c) 2007-2017 by The University of Queensland ##
## Centre for Geoscience Computing                         ##
## http://earth.uq.edu.au/centre-geoscience-computing      ##
##                                                         ##
## Primary Business: Brisbane, Queensland, Australia       ##
## Licensed under the Open Software License version 3.0    ##
## http://www.apache.org/licenses/LICENSE-2.0              ##
##                                                         ##
#############################################################

from __future__ import print_function
from gengeo     import *
from read_jointset import *
import sys

# An example python script to generate a bonded rectangular prism,
# import joint sets and tag bonds crossing joints
# takes file name of joint set as an argument

# particle properties
minRadius = 0.2
maxRadius = 1.0

#set up joint set
joints=TriPatchSet()

# read joints from file
readJointFileTagSet(joints,sys.argv[1])

# output bounding box of joint set
print("Bounding Box of Joint Set: " , joints.getMinPoint(), joints.getMaxPoint())

# Define region extremities using the bounding box of the joint set
minPoint = joints.getMinPoint()-Vector3(1.0,1.0,1.0)
maxPoint = joints.getMaxPoint()-Vector3(1.0,1.0,1.0)


# Define the volume to be filled with spheres:

box = BoxWithPlanes3D (
   minPoint = minPoint,
   maxPoint = maxPoint
)

# boundary planes
box.addPlane(Plane(minPoint,Vector3(1.0,0.0,0.0)))
box.addPlane(Plane(minPoint, Vector3(0.0,1.0,0.0)))
box.addPlane(Plane(minPoint, Vector3(0.0,0.0,1.0)))
box.addPlane(Plane(maxPoint, Vector3(-1.0,0.0,0.0)))
box.addPlane(Plane(maxPoint, Vector3(0.0,-1.0,0.0)))
box.addPlane(Plane(maxPoint, Vector3(0.0,0.0,-1.0)))

# Create a multi-group neighbour table to contain the particles:
mntable = MNTable3D (
   minPoint = minPoint,
   maxPoint = maxPoint,
   gridSize = 2.5*maxRadius
)


# Fill the volume with particles:
packer = InsertGenerator3D (
   minRadius = minRadius,
   maxRadius = maxRadius,
   insertFails = 1000,
   maxIterations = 1000,
   tolerance = 1.0e-6,
   seed=42
)

packer.generatePacking(
   volume = box, 
   ntable = mntable,
   tag = 2
)


#bonding
tol=1.0e-5
basetag=1 # tag for the "normal" bonds
mntable.generateBondsWithJointSet(joints,0,tol,basetag)

# write a geometry and a VTK file
mntable.write("temp/box_"+str(maxRadius)+".geo",1)
mntable.write("temp/box_"+str(maxRadius)+".vtu",2)
